@@ -8,7 +8,7 @@ package Tickit::Test;
 use strict;
 use warnings;
 
-our $VERSION = '0.50';
+our $VERSION = '0.51';
 
 use Carp;
 
@@ -221,9 +221,6 @@ sub resize_term
 {
    my ( $lines, $cols ) = @_;
    $term->resize( $lines, $cols );
-
-   # This is evil hackery
-   $tickit->rootwin->resize( $tickit->lines, $tickit->cols );
 }
 
 =head2 presskey( $type, $str, $mod )
