@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2015 -- leonerd@leonerd.org.uk
 
 package Tickit::Widget;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.50';
+our $VERSION = '0.51';
 
 use Carp;
 use Scalar::Util qw( weaken );
@@ -114,37 +114,25 @@ values.
 
 =cut
 
-my %warned_no_pen_for_class;
-
 sub new
 {
    my $class = shift;
    my %args = @_;
 
-   foreach my $method (qw( lines cols )) {
+   foreach my $method (qw( lines cols render_to_rb )) {
       $class->can( $method ) or
          croak "$class cannot ->$method - do you subclass and implement it?";
    }
 
-   # Require override ->render_to_rb
-   if( $class->can( "render_to_rb" ) ) {
-      # OK
-   }
-   else {
-      croak "$class cannot ->render_to_rb - do you subclass and implement it?";
-   }
+   $class->WIDGET_PEN_FROM_STYLE or
+      croak "Disabling WIDGET_PEN_FROM_STYLE is no longer allowed ($class)";
 
    my $self = bless {
       classes => delete $args{classes} // [ delete $args{class} ],
    }, $class;
 
-   if( $class->WIDGET_PEN_FROM_STYLE ) {
-      $args{$_} and $args{style}{$_} = delete $args{$_} for @Tickit::Pen::ALL_ATTRS;
-   }
-   else {
-      $warned_no_pen_for_class{$class}++ or
-         carp "Disabling WIDGET_PEN_FROM_STYLE is now deprecated for $class";
-   }
+   # Legacy direct-applied-style argument support
+   $args{$_} and $args{style}{$_} = delete $args{$_} for @Tickit::Pen::ALL_ATTRS;
 
    if( my $style = delete $args{style} ) {
       my $tagset = $self->{style_direct} = Tickit::Style::_Tagset->new;
@@ -153,12 +141,7 @@ sub new
       }
    }
 
-   if( $class->WIDGET_PEN_FROM_STYLE ) {
-      $self->set_pen( $self->get_style_pen->as_mutable );
-   }
-   else {
-      $self->set_pen( Tickit::Pen->new_from_attrs( \%args ) );
-   }
+   $self->_update_pen( $self->get_style_pen );
 
    return $self;
 }
@@ -322,28 +305,6 @@ supplied, it will be prefixed on the pen attribute names with an underscore
 (which would be read from the stylesheet file as a hypen). Note that the
 returned pen instance is immutable, and may be cached.
 
-If the class constant method C<WIDGET_PEN_FROM_STYLE> takes a true value, then
-extra logic is applied to the constructor and during style changes, to set the
-widget pen from the default style pen. Furthermore, plain attributes given to
-the constructor that take the names of pen attributes will be set on the
-widget's direct-applied style. This has the overall effect of unifying the
-widget pen with the default style pen, and additionally allowing further
-customisation for state changes or style classes.
-
-This behaviour will become the default in a future version with the eventual
-aim to remove the idea of a widget pen entirely. If a widget is constructed
-with this false it will yield a warning.
-
- use constant WIDGET_PEN_FROM_STYLE => 1;
-
-The widget pen is set to be a mutable clone of the default style pen, to allow
-the legacy behaviour that some code may attempt to mutate the widget pen
-directly. In this case the widget's direct-applied style will not be updated
-to reflect the changes, however. Code using widgets with style-managed pens
-should not attempt to mutate the widget pen, but should use C<set_style>
-instead. A future version may yield warnings or exceptions if the
-style-managed widget pen is mutated.
-
 =cut
 
 sub get_style_pen
@@ -463,8 +424,8 @@ sub _style_changed_values
       }
    }
 
-   if( $changed_pens{""} and $self->WIDGET_PEN_FROM_STYLE ) {
-      $self->set_pen( $self->get_style_pen->as_mutable );
+   if( $changed_pens{""} ) {
+      $self->_update_pen( $self->get_style_pen );
    }
 
    my $reshape = 0;
@@ -566,7 +527,7 @@ sub window_gained
       my ( $win, $rb, $rect ) = @_;
       $win->is_visible or return;
 
-      $rb->setpen( $self->pen );
+      $rb->setpen( $self->{pen} );
 
       $self->render_to_rb( $rb, $rect );
    });
@@ -779,39 +740,41 @@ sub redraw
 
 =head2 $pen = $widget->pen
 
-Returns the widget's L<Tickit::Pen>. Modifying an attribute of the returned
-object results in the widget being redrawn if the widget has a window
-associated.
+Returns the widget's L<Tickit::Pen>.
+
+Currently this returns a mutable pen, and modifying an attribute of the
+returned object results in the widget being redrawn if the widget has a window
+associated. This behaviour is now deprecated; all style modifications should
+be performed instead using the style system.
 
 =cut
 
 sub pen
 {
    my $self = shift;
+
+   unless( $self->{pen}->mutable ) {
+      $self->_update_pen( $self->{pen}->as_mutable );
+   }
+
    return $self->{pen};
 }
 
-=head2 $widget->set_pen( $pen )
-
-Set a new C<Tickit::Pen> object. This is stored by reference; changes to the
-pen will be reflected in the rendered look of the widget. The same pen may be
-shared by more than one widget; updates will affect them all.
-
-=cut
-
-sub set_pen
+sub _update_pen
 {
    my $self = shift;
    my ( $newpen ) = @_;
 
-   croak ref($self)." uses Tickit::Style for its widget pen; ->set_pen cannot be used"
-      if caller ne __PACKAGE__ and $self->WIDGET_PEN_FROM_STYLE;
-
    return if $self->{pen} and $self->{pen} == $newpen;
 
-   $self->{pen}->remove_on_changed( $self ) if $self->{pen} and $self->{pen}->mutable;
-   $self->{pen} = $newpen;
-   $newpen->add_on_changed( $self ) if $newpen->mutable;
+   {
+      no warnings 'deprecated';
+
+      $self->{pen}->remove_on_changed( $self ) if $self->{pen} and $self->{pen}->mutable;
+      $self->{pen} = $newpen;
+
+      $newpen->add_on_changed( $self ) if $newpen->mutable;
+   }
 
    if( $self->window ) {
       $self->window->set_pen( $newpen );
@@ -1023,8 +986,7 @@ change the pen foreground colour.
     my ( $args ) = @_;
 
     if( $args->type eq "text" and $args->str =~ m/[0-7]/ ) {
-       $self->pen->chattr( fg => $args->str );
-       $self->redraw;
+       $self->set_style( fg => $args->str );
        return 1;
     }
 
