@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2015 -- leonerd@leonerd.org.uk
 
 package Tickit::Window;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use 5.010; # //
 
-our $VERSION = '0.50';
+our $VERSION = '0.51';
 
 use Carp;
 
@@ -25,10 +25,6 @@ use Tickit::Utils qw( string_countmore );
 use Tickit::Debug;
 
 use constant WEAKEN_CHILDREN => $ENV{TICKIT_CHILD_WINDOWS_WEAKEN} // 1;
-use constant CHILD_WINDOWS_LATER => $ENV{TICKIT_CHILD_WINDOWS_LATER} // 1;
-unless( CHILD_WINDOWS_LATER ) {
-   die "Tickit::Window cannot run with CHILD_WINDOWS_LATER disabled\n";
-}
 
 =head1 NAME
 
@@ -91,13 +87,7 @@ are above it.
 In order to minimise the chances of ordering-specific bugs in window event
 handlers that cause child window creation, reordering or deletion, the actual
 child window list is only mutated after the event processing has finished, by
-using a L<Tickit> C<later> block. As this behaviour is relatively new, it may
-result in bugs in legacy code that isn't expecting it. For now, it can be
-disabled by setting the environment variable C<TICKIT_CHILD_WINDOWS_LATER> to
-a false value; though this is now deprecated, and will be removed in a later
-version. It will now warn on startup to remind you it is disabled.
-
- $ TICKIT_CHILD_WINDOWS_LATER=0 perl my-program.pl
+using a L<Tickit> C<later> block.
 
 =cut
 
@@ -105,7 +95,7 @@ version. It will now warn on startup to remind you it is disabled.
 sub new
 {
    my $class = shift;
-   my ( $tickit, $lines, $cols ) = @_;
+   my ( $tickit ) = @_;
 
    my $term = $tickit->term;
 
@@ -114,13 +104,21 @@ sub new
       term    => $term,
       top     => 0,
       left    => 0,
-      cols    => $cols,
-      lines   => $lines,
+      cols    => $term->cols,
+      lines   => $term->lines,
       # Only root window now accumulates damage
       damage  => Tickit::RectSet->new,
    }, $class;
    $self->_init;
 
+   weaken( my $weakself = $self );
+   $self->{bind_resize_id} = $term->bind_event( resize => sub {
+      $weakself or return;
+      my ( $term, $ev, $args ) = @_;
+      Tickit::Debug->log( Ir => "Resize to $args->{cols}x$args->{lines}" ) if DEBUG;
+      $weakself->resize( $args->{lines}, $args->{cols} );
+   } );
+
    weaken( $self->{tickit} );
 
    return $self;
@@ -237,6 +235,10 @@ sub DESTROY
    my $self = shift;
    $self->_close;
    $self->{parent}->_do_change_children( remove => $self ) if $self->{parent};
+
+   if( my $term = $self->{term} ) {
+      $term->unbind_event_id( $self->{bind_resize_id} );
+   }
 }
 
 sub close
@@ -283,14 +285,9 @@ sub _change_children
    my $self = shift;
    my @change = @_;
 
-   if( CHILD_WINDOWS_LATER ) {
-      my $root = $self->root;
-      push @{ $root->{needs_geom_change} }, [ $self => @change ];
-      $root->_needs_later;
-   }
-   else {
-      $self->_do_change_children( @change );
-   }
+   my $root = $self->root;
+   push @{ $root->{needs_geom_change} }, [ $self => @change ];
+   $root->_needs_later;
 }
 
 =head2 $sub = $win->make_sub( $top, $left, $lines, $cols )
